@@ -5,15 +5,11 @@ For an example config file, see the manual. If you don't have the man page
 installed, a raw version is available in doc/udiskie.8.txt.
 """
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import logging
 import os
 import fnmatch
 
 from .common import exc_message
-from .compat import basestring, fix_str_conversions
 from .locale import _
 
 
@@ -32,24 +28,12 @@ def lower(s):
 def match_value(value, pattern):
     if isinstance(value, (list, tuple)):
         return any(match_value(v, pattern) for v in value)
-    if isinstance(value, basestring) and isinstance(pattern, basestring):
+    if isinstance(value, str) and isinstance(pattern, str):
         return fnmatch.fnmatch(value.lower(), pattern.lower())
     return lower(value) == lower(pattern)
 
 
-def yaml_load(stream):
-    """Load YAML document, but load all strings as unicode on py2."""
-    import yaml
-    class UnicodeLoader(yaml.SafeLoader):
-        pass
-    UnicodeLoader.add_constructor(
-        yaml.resolver.BaseResolver.DEFAULT_SCALAR_TAG,
-        UnicodeLoader.construct_scalar)
-    return yaml.load(stream, UnicodeLoader)
-
-
-@fix_str_conversions
-class DeviceFilter(object):
+class DeviceFilter:
 
     """Associate a certain value to matching devices."""
 
@@ -67,6 +51,7 @@ class DeviceFilter(object):
         'has_media',
         'device_file',
         'device_presentation',
+        'device_size',
         'device_id',
         'id_usage',
         'is_crypto',
@@ -98,19 +83,14 @@ class DeviceFilter(object):
     ]
 
     def __init__(self, match):
-        """
-        Construct an instance.
-
-        :param dict match: device attributes
-        :param list value: value
-        """
+        """Construct from dict of device matching attributes."""
         self._log = logging.getLogger(__name__)
         self._match = match = match.copy()
         self._values = {}
         # mount options:
         if 'options' in match:
             options = match.pop('options')
-            if isinstance(options, basestring):
+            if isinstance(options, str):
                 options = [o.strip() for o in options.split(',')]
             self._values['options'] = options
         # ignore device:
@@ -121,7 +101,10 @@ class DeviceFilter(object):
             self._values['automount'] = match.pop('automount')
         # keyfile:
         if 'keyfile' in match:
-            self._values['keyfile'] = match.pop('keyfile')
+            keyfile = match.pop('keyfile')
+            keyfile = os.path.expandvars(keyfile)
+            keyfile = os.path.expanduser(keyfile)
+            self._values['keyfile'] = keyfile
         # the use of list() makes deletion inside the loop safe:
         for k in list(self._match):
             if k not in self.VALID_PARAMETERS:
@@ -136,11 +119,7 @@ class DeviceFilter(object):
                  self._values)
 
     def match(self, device):
-        """
-        Check if the device matches this filter.
-
-        :param Device device: device to be checked
-        """
+        """Check if the device object matches this filter."""
         return all(match_value(getattr(device, k), v)
                    for k, v in self._match.items())
 
@@ -149,9 +128,7 @@ class DeviceFilter(object):
 
     def value(self, kind, device):
         """
-        Get the associated value.
-
-        :param Device device: matched device
+        Get the value for the device object associated with this filter.
 
         If :meth:`match` is False for the device, the return value of this
         method is undefined.
@@ -169,9 +146,8 @@ class MountOptions(DeviceFilter):
     """Associate a list of mount options to matched devices."""
 
     def __init__(self, config_item):
-        """Parse the MountOptions filter from the config item."""
         config_item.setdefault('options', None)
-        super(MountOptions, self).__init__(config_item)
+        super().__init__(config_item)
 
 
 class IgnoreDevice(DeviceFilter):
@@ -179,18 +155,18 @@ class IgnoreDevice(DeviceFilter):
     """Associate a boolean ignore flag to matched devices."""
 
     def __init__(self, config_item):
-        """Parse the IgnoreDevice filter from the config item."""
         config_item.setdefault('ignore', True)
-        super(IgnoreDevice, self).__init__(config_item)
+        super().__init__(config_item)
 
 
 def match_config(filters, device, kind, default):
     """
     Matches devices against multiple :class:`DeviceFilter`s.
 
-    :param default: default value
     :param list filters: device filters
     :param Device device: device to be mounted
+    :param str kind: value kind
+    :param default: default value
     :returns: value of the first matching filter
     """
     if device is None:
@@ -201,25 +177,17 @@ def match_config(filters, device, kind, default):
     return next(matches, default)
 
 
-class Config(object):
+class Config:
 
     """Udiskie config in memory representation."""
 
     def __init__(self, data):
-        """
-        Initialize with preparsed data object.
-
-        :param ConfigParser data: config file accessor
-        """
+        """Initialize with preparsed data dict."""
         self._data = data or {}
 
     @classmethod
     def default_pathes(cls):
-        """
-        Return the default config file pathes.
-
-        :rtype: list
-        """
+        """Return the default config file pathes as a list."""
         try:
             from xdg.BaseDirectory import xdg_config_home as config_home
         except ImportError:
@@ -230,11 +198,8 @@ class Config(object):
     @classmethod
     def from_file(cls, path=None):
         """
-        Read config file.
+        Read YAML config file. Returns Config object.
 
-        :param str path: YAML config file name
-        :returns: configuration object
-        :rtype: Config
         :raises IOError: if the path does not exist
         """
         # None => use default
@@ -255,7 +220,7 @@ class Config(object):
         if os.path.splitext(path)[1].lower() == '.json':
             from json import load
         else:
-            load = yaml_load
+            from yaml import safe_load as load
         with open(path) as f:
             return cls(load(f))
 
@@ -285,3 +250,8 @@ class Config(object):
     def notification_actions(self):
         """Get the notification actions dictionary from the config file."""
         return self._data.get('notification_actions', {}).copy()
+
+    @property
+    def quickmenu_actions(self):
+        """Get the set of actions to be shown in the quickmenu (left-click)."""
+        return self._data.get('quickmenu_actions', None)
