@@ -6,22 +6,17 @@ Requires UDisks2 2.1.1 as described here:
 
     http://udisks.freedesktop.org/docs/latest
 
-This wraps the DBus API of Udisks2 providing a common interface with the
-udisks1 module.
+This wraps the DBus API of Udisks2.
 """
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 from copy import copy, deepcopy
 import logging
 
 from gi.repository import GLib
 
-from .common import Emitter, AttrDictView, decode_ay, BaseDevice
-from .dbus import connect_service, MethodsProxy, DBusCallWithFdList, DBusCall
+import udiskie.dbus as dbus
+from .common import Emitter, AttrDictView, decode_ay, samefile, sameuuid
 from .locale import _
-from .async_ import Coroutine, Return
 
 __all__ = ['Daemon']
 
@@ -30,7 +25,10 @@ def object_kind(object_path):
     """
     Parse the kind of object from an UDisks2 object path.
 
-    Example: /org/freedesktop/UDisks2/block_devices/sdb1 => device
+    Example:
+        /org/freedesktop/UDisks2/block_devices/sdb1 => device
+        /org/freedesktop/UDisks2/drives/WDC_WD...   => drive
+        /org/freedesktop/UDisks2/jobs/5             => job
     """
     try:
         return {
@@ -69,7 +67,7 @@ Interface = {
 # Internal helper classes
 # ----------------------------------------
 
-class MethodHub(object):
+class MethodHub:
 
     """Provide MethodsProxies for queried interfaces of a DBus object."""
 
@@ -79,10 +77,10 @@ class MethodHub(object):
 
     def __getattr__(self, key):
         """Return a MethodsProxy for the requested interface."""
-        return MethodsProxy(self._object_proxy, Interface[key])
+        return dbus.MethodsProxy(self._object_proxy, Interface[key])
 
 
-class PropertyHub(object):
+class PropertyHub:
 
     """Provide attribute accessors for queried interfaces of a DBus object."""
 
@@ -99,13 +97,12 @@ class PropertyHub(object):
             return PropertiesNotAvailable()
 
 
-class PropertiesNotAvailable(object):
+class PropertiesNotAvailable:
 
     """Null class for properties of an unavailable interface."""
 
-    def __nonzero__(self):      # python2
+    def __bool__(self):
         return False
-    __bool__ = __nonzero__      # python3
 
     def __getattr__(self, key):
         """Return None when asked for any attribute."""
@@ -116,7 +113,7 @@ class PropertiesNotAvailable(object):
 # Device wrapper
 # ----------------------------------------
 
-class Device(BaseDevice):
+class Device:
 
     """
     Proxy class for UDisks2 devices.
@@ -133,6 +130,18 @@ class Device(BaseDevice):
         self._P = property_hub
         self._M = method_hub
 
+    def __str__(self):
+        """Show as object_path."""
+        return self.object_path
+
+    def __eq__(self, other):
+        """Comparison by object_path."""
+        return self.object_path == str(other)
+
+    def __ne__(self, other):
+        """Comparison by object_path."""
+        return not (self == other)
+
     # availability of interfaces
     @property
     def is_drive(self):
@@ -327,7 +336,7 @@ class Device(BaseDevice):
         """Check if the device is external."""
         # NOTE: Checking for equality HintSystem==False returns False if the
         # property is resolved to a None value (interface not available).
-        if self._P.Block.HintSystem == False:
+        if self._P.Block.HintSystem == False:       # noqa: E712
             return True
         # NOTE: udisks2 seems to guess incorrectly in some cases. This
         # leads to HintSystem=True for unlocked devices. In order to show
@@ -358,9 +367,7 @@ class Device(BaseDevice):
 
     @property
     def root(self):
-        """
-        Get the top level block device in the ancestry of this device.
-        """
+        """Get the top level block device in the ancestry of this device."""
         drive = self.drive
         for device in self._daemon:
             if device.is_drive:
@@ -532,18 +539,83 @@ class Device(BaseDevice):
             })
         )
 
-    loop_support = True
-
     # ----------------------------------------
     # derived properties
     # ----------------------------------------
 
+    def is_file(self, path):
+        """Comparison by mount and device file path."""
+        return (samefile(path, self.device_file) or
+                samefile(path, self.loop_file) or
+                any(samefile(path, mp) for mp in self.mount_paths) or
+                sameuuid(path, self.id_uuid) or
+                sameuuid(path, self.partition_uuid))
+
     @property
     def parent_object_path(self):
         return (self._P.Partition.Table
                 or self._P.Block.CryptoBackingDevice
                 or '/')
 
+    @property
+    def mount_path(self):
+        """Return any mount path."""
+        try:
+            return self.mount_paths[0]
+        except IndexError:
+            return ''
+
+    @property
+    def in_use(self):
+        """Check whether this device is in use, i.e. mounted or unlocked."""
+        if self.is_mounted or self.is_unlocked:
+            return True
+        if self.is_partition_table:
+            for device in self._daemon:
+                if device.partition_slave == self and device.in_use:
+                    return True
+        return False
+
+    @property
+    def ui_id_label(self):
+        """Label of the unlocked partition or the device itself."""
+        return (self.luks_cleartext_holder or self).id_label
+
+    @property
+    def ui_id_uuid(self):
+        """UUID of the unlocked partition or the device itself."""
+        return (self.luks_cleartext_holder or self).id_uuid
+
+    @property
+    def ui_device_presentation(self):
+        """Path of the crypto backing device or the device itself."""
+        return (self.luks_cleartext_slave or self).device_presentation
+
+    @property
+    def ui_label(self):
+        """UI string identifying the partition if possible."""
+        return ': '.join(filter(None, [
+            self.ui_device_presentation,
+            self.ui_id_label or self.ui_id_uuid or self.drive_label
+        ]))
+
+    @property
+    def ui_device_label(self):
+        """UI string identifying the device (drive) if toplevel."""
+        return ': '.join(filter(None, [
+            self.ui_device_presentation,
+            self.loop_file or
+            self.drive_label or self.ui_id_label or self.ui_id_uuid
+        ]))
+
+    @property
+    def drive_label(self):
+        """Return drive label."""
+        return ' '.join(filter(None, [
+            self.drive_vendor,
+            self.drive_model,
+        ]))
+
 
 # ----------------------------------------
 # UDisks2 service wrapper
@@ -590,7 +662,7 @@ class Daemon(Emitter):
                 self._log.debug(_('found device owning "{0}": "{1}"',
                                   path, device))
                 return device
-        raise ValueError(_('no device found owning "{0}"', path))
+        raise FileNotFoundError(_('no device found owning "{0}"', path))
 
     def __init__(self, proxy, version):
 
@@ -606,14 +678,14 @@ class Daemon(Emitter):
                        'device_locked',
                        'device_changed',
                        'job_failed']
-        super(Daemon, self).__init__(event_names)
+        super().__init__(event_names)
 
         self._log = logging.getLogger(__name__)
         self._log.debug(_('Daemon version: {0}', version))
 
         self.version = version
         self.version_info = tuple(map(int, version.split('.')))
-        self.keyfile_support = self.version_info >= (2,6,4)
+        self.keyfile_support = self.version_info >= (2, 6, 4)
         self._log.debug(_('Keyfile support: {0}', self.keyfile_support))
 
         self._proxy = proxy
@@ -632,45 +704,39 @@ class Daemon(Emitter):
                     None,
                     self._job_completed)
 
-    def _sync(self):
+    async def _sync(self):
         """Synchronize state."""
-        def update_objects(objects):
-            self._objects = objects
-        update = self._proxy.call('GetManagedObjects', '()')
-        update.callbacks.append(update_objects)
-        return update
+        self._objects = await self._proxy.call('GetManagedObjects', '()')
 
     @classmethod
-    @Coroutine.from_generator_function
-    def create(cls):
+    async def create(cls):
         service = (cls.BusName, cls.ObjectPath, cls.Interface)
-        proxy = yield connect_service(*service)
-        version = yield cls.get_version()
+        proxy = await dbus.connect_service(*service)
+        version = await cls.get_version()
         daemon = cls(proxy, version)
-        yield daemon._sync()
-        yield Return(daemon)
+        await daemon._sync()
+        return daemon
 
     @classmethod
-    @Coroutine.from_generator_function
-    def get_version(cls):
+    async def get_version(cls):
         service = (cls.BusName,
                    '/org/freedesktop/UDisks2/Manager',
                    Interface['Properties'])
-        manager = yield connect_service(*service)
-        version = yield DBusCall(manager._proxy, 'Get', '(ss)', (
+        manager = await dbus.connect_service(*service)
+        version = await dbus.call(manager._proxy, 'Get', '(ss)', (
             Interface['Manager'], 'Version'))
-        yield Return(version)
+        return version
 
-    @Coroutine.from_generator_function
-    def loop_setup(self, fd, options):
+    async def loop_setup(self, fd, options):
         service = (self.BusName,
                    '/org/freedesktop/UDisks2/Manager',
                    Interface['Manager'])
-        manager = yield connect_service(*service)
-        object_path = yield DBusCallWithFdList(
+        manager = await dbus.connect_service(*service)
+        object_path = await dbus.call_with_fd_list(
             manager._proxy, 'LoopSetup', '(ha{sv})',
             (0, filter_opt({
-                'auth.no_user_interaction': ('b', options.get('auth.no_user_interaction')),
+                'auth.no_user_interaction': (
+                    'b', options.get('auth.no_user_interaction')),
                 'offset': ('t', options.get('offset')),
                 'size': ('t', options.get('size')),
                 'read-only': ('b', options.get('read-only')),
@@ -678,10 +744,8 @@ class Daemon(Emitter):
             })),
             [fd],
         )
-        yield self._sync()
-        yield Return(self[object_path])
-
-    loop_support = True
+        await self._sync()
+        return self[object_path]
 
     # UDisks2 interface
     def paths(self):
@@ -702,7 +766,7 @@ class Daemon(Emitter):
 
     def trigger(self, event, device, *args):
         self._log.debug(_("+++ {0}: {1}", event, device))
-        super(Daemon, self).trigger(event, device, *args)
+        super().trigger(event, device, *args)
 
     # add objects / interfaces
     def _interfaces_added(self, object_path, interfaces_and_properties):
